import Preferences from "./Preferences.js";

export default class ItemHTML {

    static SUCCESS_CLASS_NAME = 'success';
    static WARNING_CLASS_NAME = 'warning';
    static ERROR_CLASS_NAME = 'error';

    static CONTAINER_CLASS_NAME = 'midefos-idealista-container';
    static INFORMATION_CONTAINER_CLASS_NAME = 'midefos-idealista-information-container';
    static get CONTAINER_SELECTOR() {
        return `.${this.CONTAINER_CLASS_NAME}`;
    }

    static INFORMATION_CLASS_NAME = 'information'
    static get INFORMATION_SELECTOR() {
        return `.${this.INFORMATION_CLASS_NAME}`;
    }

    static ITEM_STATE_CLASS_NAME(item) {
        const desiredPrice = Preferences.get('max-price');
        if (desiredPrice && !item.isProperPrice(desiredPrice)) {
            return this.ERROR_CLASS_NAME;
        }

        const desiredPricePerMeter = Preferences.get('max-price-per-meter');
        if (desiredPricePerMeter && !item.isProperPriceMeter(desiredPricePerMeter)) {
            return this.ERROR_CLASS_NAME;
        }

        if (this._shouldCheckGarage() && !item.hasGarage) {
            return this.ERROR_CLASS_NAME;
        }

        if (this._shouldCheckLift(item) && !item.hasLift) {
            return this.ERROR_CLASS_NAME;
        }

        if (this._shouldCheckExterior(item) && !item.isExterior) {
            return this.ERROR_CLASS_NAME;
        }
        return this.SUCCESS_CLASS_NAME;
    }

    static createInformation(item) {
        return `
        <div class='${this.CONTAINER_CLASS_NAME} ${this.ITEM_STATE_CLASS_NAME(item)}'>
            <div class='${this.INFORMATION_CONTAINER_CLASS_NAME}'>
                ${this._createPercentagePriceHTML(item)}
                ${this._createPriceHTML(item)}
                ${this._createPriceMeterHTML(item)}
                ${this._createGarageHTML(item)}
                ${this._createLiftHTML(item)}
                ${this._createExteriorHTML(item)}
            </div>
            <div class='${this.INFORMATION_CONTAINER_CLASS_NAME}'>
                ${this._createLocationInformation(item)}
            </div>
        </div>`
    }

    static _createPercentagePriceHTML(item) {
        if (!Preferences.get('percentages')) return ``;


        let html = `<div class='${this.INFORMATION_CLASS_NAME}'>`;
        if (Preferences.get('percentages_20')) {
            const twentyPercent = Math.round(item.price * 20 / 100);
            html += this._createIndividual('20%', twentyPercent);
        }

        if (Preferences.get('percentages_30')) {
            const thirtyPercent = Math.round(item.price * 30 / 100);
            html += this._createIndividual('30%', thirtyPercent)
        }

        if (Preferences.get('percentages_50')) {
            const fiftyPercent = Math.round(item.price * 50 / 100);
            html += this._createIndividual('50%', fiftyPercent)
        }
        html += `</div>`;

        return html;
    }

    static _createPriceHTML(item) {
        const desiredPrice = Preferences.get('max-price');
        if (!desiredPrice) return ``;

        const desiredTwentyFivePercentMore = Math.round(desiredPrice * 1.25);

        if (item.isProperPrice(desiredPrice)) {
            return this._createSuccess('Precio');
        } else if (item.isProperPrice(desiredTwentyFivePercentMore)) {
            return this._createWarning('Precio');
        }
        return this._createError('Precio');
    }

    static _createPriceMeterHTML(item) {
        const desiredPricePerMeter = Preferences.get('max-price-per-meter');
        if (!desiredPricePerMeter) return this._createIndividual('m²', item.priceMeter);

        const desiredTwentyFivePercentMore = Math.round(desiredPricePerMeter * 1.25);

        if (item.priceMeter <= desiredPricePerMeter) {
            return this._createTextSuccess('m²', item.priceMeter)
        } else if (item.priceMeter <= desiredTwentyFivePercentMore) {
            return this._createTextWarning('m²', item.priceMeter)
        }
        return this._createTextError('m²', item.priceMeter)
    }

    static _createGarageHTML(item) {
        if (!this._shouldCheckGarage()) {
            return ``;
        }

        if (item.hasGarage) {
            return this._createSuccess('Garaje');
        }
        return this._createError('Garaje');
    }

    static _shouldCheckGarage() {
        return Preferences.get('garage');
    }

    static _createLiftHTML(item) {
        if (!this._shouldCheckLift(item)) {
            return ``;
        }

        if (!item.additionalInfo) {
            return this._createMissing('Ascensor');
        }

        if (item.hasLift) {
            return this._createSuccess('Ascensor');
        }
        return this._createError('Ascensor');
    }

    static _shouldCheckLift(item) {
        return Preferences.get('lift')
            && !item.isHouse()
            && !item.isGround();
    }

    static _createExteriorHTML(item) {
        if (!this._shouldCheckExterior(item)) {
            return ``;
        }

        if (item.isExterior) {
            return this._createSuccess('Exterior');
        }

        if (item.isInterior) {
            return this._createError('Exterior');
        }
        return this._createMissing('Exterior');
    }

    static _createLocationInformation(item) {
        return this._createNeutral(item.locationName);
    }

    static _shouldCheckExterior(item) {
        return Preferences.get('exterior')
            && !item.isHouse();
    }

    static _createSuccess(infoText) {
        return this._createTextSuccess('✔️', infoText);
    }

    static _createTextSuccess(strongText, infoText) {
        return this._createIndividual(strongText, infoText, this.SUCCESS_CLASS_NAME);
    }

    static _createWarning(infoText) {
        return this._createTextWarning('⚠️', infoText);
    }

    static _createMissing(infoText) {
        return this._createTextWarning('?', infoText);
    }

    static _createTextWarning(strongText, infoText) {
        return this._createIndividual(strongText, infoText, this.WARNING_CLASS_NAME);
    }

    static _createError(infoText) {
        return this._createTextError('🚫', infoText);
    }

    static _createTextError(strongText, infoText) {
        return this._createIndividual(strongText, infoText, this.ERROR_CLASS_NAME);
    }

    static _createNeutral(infoText) {
        return this._createIndividual('ⓘ', infoText)
    }

    static _createIndividual(strongText, infoText, className = '') {
        return `<span class='${className}'><strong>${strongText}</strong> ${infoText}</span>`;
    }

}